// =============================================================================
//                           COPYRIGHT NOTICE
// Copyright 2010 (c) Lattice Semiconductor Corporation
// ALL RIGHTS RESERVED
// This confidential and proprietary software may be used only as authorised by
// a licensing agreement from Lattice Semiconductor Corporation.
// The entire notice above must be reproduced on all authorized copies and
// copies may only be made to the extent permitted by a licensing agreement from
// Lattice Semiconductor Corporation.
//
// Lattice Semiconductor Corporation        TEL : 1-800-Lattice (USA and Canada)
// 5555 NE Moore Court                            408-826-6000 (other locations)
// Hillsboro, OR 97124                      web  : http://www.latticesemi.com/
// U.S.A                                    email: techsupport@latticesemi.com
// =============================================================================
//                         FILE DETAILS
// Project          : 
// File             :
// Title            : 
// Dependencies     :
// Description      :
// =============================================================================
//                        REVISION HISTORY
// Version          : 
// Author(s)        :  
// Mod. Date        : 
// Changes Made     : Initial Creation
// =============================================================================
//
//
//
// ECP3-35EA Versa Board Mico32 Testing
//
// FPGA: LFE3-35EA-FF484
//
// 8 DIP Switchs
// 8 user LEDs
// 14 segment display + decimal point
// USB UART
// SDRAM interface to I/O expansion connectors
//
// brd_clk = 100 MHz (LVDS on pin L5)
// Sys_clk = 50 MHz (brd_clk / 2)
//

`timescale 1 ps / 1 ps 

`include "../MSB/soc/system_conf.v"   // MSB definitions


module top_Versa (


	input wire	reset_n,

	input wire	brd_clk/*synthesis syn_keep = 1*/,
        input wire	usb_uart_txd, 
	output wire	usb_uart_rxd,

        input wire [7:0] dip_switch,        // logic/position: 0=on/down (grounded), 1=off/up (open)
	output wire [7:0] led,              // negative logic 0=on, 1=off    
	output wire [13:0] seg,              // negative logic 0=on, 1=off    
	output wire 	dp,              // negative logic 0=on, 1=off    
	input  wire	in_hdw,         // 1 = running on board, 0 = simulating
	inout  wire     i2cm_sda,
	input  wire     i2cm_scl

);

	




	wire sys_clk /*synthesis syn_keep = 1*/;
	wire sys_clk_half /*synthesis syn_keep = 1*/;
	wire [7:0] led_inv; 
	wire [15:0] seg_inv;   // Still use old 16 segment driver routines, but convert to 14 segment Versa here
	reg [23:0] HB_cntr;

	reg [3:0] rst_cntr;
	reg 	  cpu_rst_n;



`ifdef SIMULATION
     	// In simulation, leave signals as positive logic so it reads normal in simulation waves and
	// test bench settings
	assign led = led_inv; 
	// Convert the 16 segment charcter code into the new 14 segment display on the Versa board.
	assign	seg = {seg_inv[15:6], seg_inv[5] | seg_inv[4], seg_inv[3:2], seg_inv[1] | seg_inv[0]} ;

	assign dp =  HB_cntr[4];   // faster blink rate to see in sim
`else
     	// In real hardware the signals are negative logic to light LED
	assign led = ~led_inv;
	// Convert the 16 segment charcter code into the new 14 segment display on the Versa board.
	assign	seg = ~{seg_inv[15:6], seg_inv[5] | seg_inv[4], seg_inv[3:2], seg_inv[1] | seg_inv[0]} ;

	assign dp = HB_cntr[23];
`endif



    // Create some divided clocks from the 100 MHz on board clock
    // Easier to meet timing with 50 Mhz Mico clock.  No need to run at 100
    // MHz.
    CLKDIVB divclk(
	.CLKI(brd_clk),         // from external clock source
	.RST(1'b0), 
	.RELEASE(1'b1), 
	.CDIV1(), 
	.CDIV2(sys_clk),        // Mico system clock = 50 MHz
	.CDIV4(sys_clk_half),   // slower clock for heart beat counter
	.CDIV8()
    );



   // 25 Mhz counter for Heartbeat LED
   // blink ~ 1/4 sec 
   always @ (posedge sys_clk_half  or negedge reset_n)
   begin
	if  (reset_n == 0)
	 	HB_cntr <= 24'h00_0000;
 	else
		HB_cntr <= HB_cntr + 1;
    end

    // Generate a Mico system reset to hold LM32 in reset for some clocks
   always @ (posedge sys_clk  or negedge reset_n)
   begin
	if  (reset_n == 0)
	begin
	 	rst_cntr <= 4'h0;
		cpu_rst_n <= 0;
	end
 	else
	begin
		rst_cntr <= rst_cntr + 1;
		if ((cpu_rst_n == 0) && (rst_cntr == 4'hf))
			cpu_rst_n <= 1;
	end
    end



    /*
    * Instantiate the Mico32 Platform
    * UART tx/rx connections to FTDI USB chip
    * GPIO_LED to 8 user LEDs on the left edge of the board
    * GPIO_14seg to 14 Segment display
    * GPIO_DIPSw from DIP switch pack + in_hdw signal (tied high or low)
    */
    MSB MSB_u ( 
	.clk_i(sys_clk),
	.reset_n(cpu_rst_n)


	, .uartSIN(usb_uart_txd) //  input from the FTDI USB chip to our UART
	, .uartSOUT(usb_uart_rxd) //  output from our UART to the FTDI chip
	
	, .gpio_ledPIO_OUT(led_inv) // [8-1:0]
	, .gpio_dipswPIO_IN({in_hdw, dip_switch}) // [9-1:0]
	, .gpio_14segPIO_OUT({seg_inv[7:0],seg_inv[15:8]}) // MSB GPIO is little endian [15:0]
	, .i2cm_ocSDA(i2cm_sda) // 
	, .i2cm_ocSCL(i2cm_scl) // 
    );


endmodule
