// I2C pins are:
//  C8 - SCL
//  B8 - SDA
//  N3 - reset
//
//
// 3.3V Power Supply Control
//--------------------------
//  N2 = EnI2CSPI
//  P2 = EnAMP
///////////////////////////////////////////
module top_XO2 
(
	//Inputs
	input rstn,

	// Outputs
	output oscclkout, 
	output rstout,    // debug
	output cntout,   // debug

	output EnI2CSPI,
	output EnAMP, 

	//New LCD
	output LCD_COM0,
	output LCD_COM1,
	output LCD_COM2,
	output LCD_COM3,

	output LCD_5,
	output LCD_6,
	output LCD_7,
	output LCD_8,
	output LCD_9,
	output LCD_10,
	output LCD_11,
	output LCD_12,

	inout  scl,
	inout  sda
);

	localparam COUNT_WIDTH = 30;

	wire				clk;
	wire				oscclk;
	wire [1:0]			LCD_COM0enc;
	wire [1:0]			LCD_COM1enc;
	wire [1:0]			LCD_COM2enc;
	wire [1:0]			LCD_COM3enc;
	wire [1:0]			LCD_5enc;
	wire [1:0]			LCD_6enc;
	wire [1:0]			LCD_7enc;
	wire [1:0]			LCD_8enc;
	wire [1:0]			LCD_9enc;
	wire [1:0]			LCD_10enc;
	wire [1:0]			LCD_11enc;
	wire [1:0]			LCD_12enc;

	wire				LCDcom;
	
	
	reg [COUNT_WIDTH-1:0]			lcdcounter;
	
	reg [6:0]			LCD1;
	reg [6:0]			LCD2;
	reg [6:0]			LCD3;
	reg [6:0]			LCD4;

	wire [3:0]               digit1;
	wire [3:0]               digit2;
	wire [3:0]               digit3;
	wire [3:0]               digit4;

	assign rstout = ~rstn;
	assign cntout = lcdcounter[14];

	assign EnI2CSPI = 1'b1;   // set to 1 to turn on 3.3v to I2C and SPI chips
	assign EnAMP    = 1'b1;   // set to 1 for voltage measuring circuit (if used)


//***********************************************************************
//
//  Internal Oscillator
//
//***********************************************************************
    defparam OSCH_inst.NOM_FREQ = "24.18" ;
	OSCH OSCH_inst(
		.STDBY(1'b0),
		.OSC(oscclk),
		.SEDSTDBY());

	assign oscclkout    = oscclk;
	assign clk	    = oscclk;


//***********************************************************************
//
//  free-run counter
//
//***********************************************************************
	always @(posedge clk or negedge rstn)
	begin
		if(rstn == 0) begin
			lcdcounter <= 0;
		end else begin
			lcdcounter <= lcdcounter + 1;
		end
	end

//***********************************************************************
//
//  output assignments
//
//***********************************************************************

	// LCD Com signals
	LCDEncoding4to1com LCD_COM0enc_inst (.clk(lcdcounter[14]), .rstn(rstn), .LCDcom({1'b1, 1'b0, 1'b0, 1'b0}), .LCDcomEncoded(LCD_COM0enc));
	PWM LCD_COM0_inst (.clk(lcdcounter[5]), .rstn(rstn), .Voltage(LCD_COM0enc), .PWMVoltage(LCD_COM0));

	LCDEncoding4to1com LCD_COM1enc_inst (.clk(lcdcounter[14]), .rstn(rstn), .LCDcom({1'b0, 1'b1, 1'b0, 1'b0}), .LCDcomEncoded(LCD_COM1enc));
	PWM LCD_COM1_inst (.clk(lcdcounter[5]), .rstn(rstn), .Voltage(LCD_COM1enc), .PWMVoltage(LCD_COM1));

	LCDEncoding4to1com LCD_COM2enc_inst (.clk(lcdcounter[14]), .rstn(rstn), .LCDcom({1'b0, 1'b0, 1'b1, 1'b0}), .LCDcomEncoded(LCD_COM2enc));
	PWM LCD_COM2_inst (.clk(lcdcounter[5]), .rstn(rstn), .Voltage(LCD_COM2enc), .PWMVoltage(LCD_COM2));

	LCDEncoding4to1com LCD_COM3enc_inst (.clk(lcdcounter[14]), .rstn(rstn), .LCDcom({1'b0, 1'b0, 1'b0, 1'b1}), .LCDcomEncoded(LCD_COM3enc));
	PWM LCD_COM3_inst (.clk(lcdcounter[5]), .rstn(rstn), .Voltage(LCD_COM3enc), .PWMVoltage(LCD_COM3));

	// LCD Segment Signals
	LCDEncoding4to1 LCD_5enc_inst (.clk(lcdcounter[14]), .rstn(rstn), .LCDSegments({!LCD1[3], !LCD1[4], !LCD1[6], !LCD1[5]}), .LCDcom(LCDcom), .LCDSegEncoded(LCD_5enc));
	PWM LCD_5_inst (.clk(lcdcounter[5]), .rstn(rstn), .Voltage(LCD_5enc), .PWMVoltage(LCD_5));

	LCDEncoding4to1 LCD_6enc_inst (.clk(lcdcounter[14]), .rstn(rstn), .LCDSegments({1'b0, !LCD1[2], !LCD1[1], !LCD1[0]}), .LCDcom(LCDcom), .LCDSegEncoded(LCD_6enc));
	PWM LCD_6_inst (.clk(lcdcounter[5]), .rstn(rstn), .Voltage(LCD_6enc), .PWMVoltage(LCD_6));

	LCDEncoding4to1 LCD_7enc_inst (.clk(lcdcounter[14]), .rstn(rstn), .LCDSegments({!LCD2[3], !LCD2[4], !LCD2[6], !LCD2[5]}), .LCDcom(LCDcom), .LCDSegEncoded(LCD_7enc));
	PWM LCD_7_inst (.clk(lcdcounter[5]), .rstn(rstn), .Voltage(LCD_7enc), .PWMVoltage(LCD_7));

	LCDEncoding4to1 LCD_8enc_inst (.clk(lcdcounter[14]), .rstn(rstn), .LCDSegments({1'b0, !LCD2[2], !LCD2[1], !LCD2[0]}), .LCDcom(LCDcom), .LCDSegEncoded(LCD_8enc));
	PWM LCD_8_inst (.clk(lcdcounter[5]), .rstn(rstn), .Voltage(LCD_8enc), .PWMVoltage(LCD_8));

	LCDEncoding4to1 LCD_9enc_inst (.clk(lcdcounter[14]), .rstn(rstn), .LCDSegments({!LCD3[3], !LCD3[4], !LCD3[6], !LCD3[5]}), .LCDcom(LCDcom), .LCDSegEncoded(LCD_9enc));
	PWM LCD_9_inst (.clk(lcdcounter[5]), .rstn(rstn), .Voltage(LCD_9enc), .PWMVoltage(LCD_9));

	LCDEncoding4to1 LCD_10enc_inst (.clk(lcdcounter[14]), .rstn(rstn), .LCDSegments({1'b0, !LCD3[2], !LCD3[1], !LCD3[0]}), .LCDcom(LCDcom), .LCDSegEncoded(LCD_10enc));
	PWM LCD_10_inst (.clk(lcdcounter[5]), .rstn(rstn), .Voltage(LCD_10enc), .PWMVoltage(LCD_10));

	LCDEncoding4to1 LCD_11enc_inst (.clk(lcdcounter[14]), .rstn(rstn), .LCDSegments({!LCD4[3], !LCD4[4], !LCD4[6], !LCD4[5]}), .LCDcom(LCDcom), .LCDSegEncoded(LCD_11enc));
	PWM LCD_11_inst (.clk(lcdcounter[5]), .rstn(rstn), .Voltage(LCD_11enc), .PWMVoltage(LCD_11));

	LCDEncoding4to1 LCD_12enc_inst (.clk(lcdcounter[14]), .rstn(rstn), .LCDSegments({1'b0, !LCD4[2], !LCD4[1], !LCD4[0]}), .LCDcom(LCDcom), .LCDSegEncoded(LCD_12enc));
	PWM LCD_12_inst (.clk(lcdcounter[5]), .rstn(rstn), .Voltage(LCD_12enc), .PWMVoltage(LCD_12));



	// LCD Hex Characters
	always @ (posedge clk or negedge rstn)
	begin
		if (rstn == 0)
		begin 
			 LCD1 <= 7'h7F; 
			 LCD2 <= 7'h7F;
			 LCD3 <= 7'h7F;
 			 LCD4 <= 7'h7F;
		end
		else
		begin
			case (digit1)
				//LCDx <=        GFEDCBA 0 turns on a segment
				4'h0: LCD1 <= 7'b1000000; // 0
				4'h1: LCD1 <= 7'b1111001; // 1
				4'h2: LCD1 <= 7'b0100100;  //2
				4'h3: LCD1 <= 7'b0110000;  //3
				4'h4: LCD1 <= 7'b0011001;  //4
				4'h5: LCD1 <= 7'b0010010;  //5
				4'h6: LCD1 <= 7'b0000010;  //6
				4'h7: LCD1 <= 7'b1111000;  //7
				4'h8: LCD1 <= 7'b0000000;  //8
				4'h9: LCD1 <= 7'b0010000;  //9
				4'hA: LCD1 <= 7'b0001000;  //A
				4'hB: LCD1 <= 7'b0000011;  //B
				4'hC: LCD1 <= 7'b1000110;  //C
				4'hD: LCD1 <= 7'b0100001;  //D
				4'hE: LCD1 <= 7'b0000110;  //E
				4'hF: LCD1 <= 7'b0001110;  //F
			endcase

			case (digit2)
				//LCDx <=        GFEDCBA 0 turns on a segment
				4'h0: LCD2 <= 7'b1000000; // 0
				4'h1: LCD2 <= 7'b1111001; // 1
				4'h2: LCD2 <= 7'b0100100;  //2
				4'h3: LCD2 <= 7'b0110000;  //3
				4'h4: LCD2 <= 7'b0011001;  //4
				4'h5: LCD2 <= 7'b0010010;  //5
				4'h6: LCD2 <= 7'b0000010;  //6
				4'h7: LCD2 <= 7'b1111000;  //7
				4'h8: LCD2 <= 7'b0000000;  //8
				4'h9: LCD2 <= 7'b0010000;  //9
				4'hA: LCD2 <= 7'b0001000;  //A
				4'hB: LCD2 <= 7'b0000011;  //B
				4'hC: LCD2 <= 7'b1000110;  //C
				4'hD: LCD2 <= 7'b0100001;  //D
				4'hE: LCD2 <= 7'b0000110;  //E
				4'hF: LCD2 <= 7'b0001110;  //F
			endcase

			case (digit3)
				//LCDx <=        GFEDCBA 0 turns on a segment
				4'h0: LCD3 <= 7'b1000000; // 0
				4'h1: LCD3 <= 7'b1111001; // 1
				4'h2: LCD3 <= 7'b0100100;  //2
				4'h3: LCD3 <= 7'b0110000;  //3
				4'h4: LCD3 <= 7'b0011001;  //4
				4'h5: LCD3 <= 7'b0010010;  //5
				4'h6: LCD3 <= 7'b0000010;  //6
				4'h7: LCD3 <= 7'b1111000;  //7
				4'h8: LCD3 <= 7'b0000000;  //8
				4'h9: LCD3 <= 7'b0010000;  //9
				4'hA: LCD3 <= 7'b0001000;  //A
				4'hB: LCD3 <= 7'b0000011;  //B
				4'hC: LCD3 <= 7'b1000110;  //C
				4'hD: LCD3 <= 7'b0100001;  //D
				4'hE: LCD3 <= 7'b0000110;  //E
				4'hF: LCD3 <= 7'b0001110;  //F
			endcase

			case (digit4)
				//LCDx <=        GFEDCBA 0 turns on a segment
				4'h0: LCD4 <= 7'b1000000; // 0
				4'h1: LCD4 <= 7'b1111001; // 1
				4'h2: LCD4 <= 7'b0100100;  //2
				4'h3: LCD4 <= 7'b0110000;  //3
				4'h4: LCD4 <= 7'b0011001;  //4
				4'h5: LCD4 <= 7'b0010010;  //5
				4'h6: LCD4 <= 7'b0000010;  //6
				4'h7: LCD4 <= 7'b1111000;  //7
				4'h8: LCD4 <= 7'b0000000;  //8
				4'h9: LCD4 <= 7'b0010000;  //9
				4'hA: LCD4 <= 7'b0001000;  //A
				4'hB: LCD4 <= 7'b0000011;  //B
				4'hC: LCD4 <= 7'b1000110;  //C
				4'hD: LCD4 <= 7'b0100001;  //D
				4'hE: LCD4 <= 7'b0000110;  //E
				4'hF: LCD4 <= 7'b0001110;  //F
			endcase

		end
	end	


//***********************************************************************
//
//  EBR to be used as LookUp Table
//
//***********************************************************************
	
lookup_RAM lcdDigits 
(
	.Clock(clk ), 
	.ClockEn(1'b1 ), 
	.Reset(~rstn ), 
	.WE(1'b0), 
`ifdef SIMULATION
	.Address({1'b0, lcdcounter[15:8]} ), 
`else
	.Address({5'b00000, lcdcounter[COUNT_WIDTH - 1:COUNT_WIDTH - 4]} ),    // 9 bit address
`endif
    	.Data(16'h0000), 
	.Q( {digit1, digit2, digit3, digit4} )   // LCD display is numbered 1..4 from left to right
);




//***********************************************************************
//
//  EFB
//
//***********************************************************************
	

	myEFB efb_u (
		.wb_clk_i(clk ), 
		.wb_rst_i(~rstn ), 
		.wb_cyc_i(1'b0 ), 
		.wb_stb_i(1'b0 ), 
	    
		.wb_we_i(1'b0 ), 
		.wb_adr_i(8'h00 ), 
		.wb_dat_i(8'h00 ), 
		.wb_dat_o( ), 
		.wb_ack_o( ), 
	    
		.i2c1_scl( scl), 
		.i2c1_sda( sda), 
		.i2c1_irqo( ), 
		.wbc_ufm_irq( )
	);



endmodule

